#!/usr/bin/python
# -*- coding: UTF-8 -*-
# editor: moofei


import os
import re
import time
from collections import Counter
import math
import sys
from datetime import datetime, timedelta

date_regex = r"?P<date>\d+"
month_regex = r"?P<month>\w+"
year_regex = r"?P<year>\d+"
day_time_regex = r"?P<day_time>\S+"

ip_regex = r"?P<ip>[\d.]*"
time_regex = r"?P<time>.+"
method_regex = r"?P<method>\S+"
request_regex = r"?P<request>\S+"
status_regex = r"?P<status>\d+"
body_bytes_regex = r"?P<body_bytes>\d+"
refer_regex = r"?P<refer>\S+"
agent_regex = r"?P<agent>.+"

def parse_time(date, month, year, log_time):
    time_str = '%s%s%s %s' %(year, month, date, log_time)
    t = time.strptime(time_str, '%Y%b%d %H:%M:%S')
    return time.strftime("%Y-%m-%d %H:%M:%S",t)

#转换字节单位
def convertBytes(bytes, lst=['B','KB','MB','GB','TB','PB']):
    i = int(math.floor(math.log(bytes, 1024)))
    if i >= len(lst):
        i = len(lst) - 1
    return ('%.2f ' + lst[i]) % (bytes/math.pow(1024, i))


class AccessLog:
    def __init__(self, path):
        """
        log_fmt = r"(%s)\s+-\s+-\s+\[(%s)/(%s)/(%s)\:(%s)\ [\S]+\]\ \"(%s)?[\s]?(%s)?.*?\"\ (%s)\ (%s)\s"
        p = re.compile(log_fmt %(ip_regex, date_regex, month_regex, year_regex, day_time_regex, method_regex, request_regex, status_regex),re.VERBOSE)
        """
        self.path = path
        self.file = open(path)
        self.records  = {}
        
        log_fmt = r'(%s)\s+-\s+-\s+\[(%s)/(%s)/(%s)\:(%s)\ [\S]+\]\ \"(%s)?[\s]?(%s)?.*?\"\ (%s)\ (%s)\s\"(%s)\"\s\"(%s)\"'
        self.p = re.compile(log_fmt %(ip_regex, date_regex, month_regex, year_regex, day_time_regex,
                                      method_regex, request_regex, status_regex,body_bytes_regex, refer_regex, agent_regex)
                            ,re.VERBOSE)

        #log_fmt = r'(%s)\s+-\s+-\s+\[(%s)]\ \"(%s)?[\s]?(%s)?.*?\"\ (%s)\ (%s)\s\"(%s)\"\s\"(%s)\"'
        #self.p = re.compile(log_fmt %(ip_regex, time_regex,
        #                              method_regex, request_regex, status_regex,body_bytes_regex, refer_regex, agent_regex)
        #                    ,re.VERBOSE)
        
    def ngx(self):
        for line in self.file:
            if not line: continue
            d = self.p.search(line)
            if not d: continue
            
            d = d.groupdict()
            #print d
            date = d["date"]
            month = d["month"]
            year = d["year"]
            day_time = d["day_time"]
            ip = d['ip']
            status = int(d['status'] or 0)
            body_bytes = int(d['body_bytes'] or 0)
            refer = d['refer']
            agent = d['agent']
            method = d['method']
            request = d['request']
            
            IP = self.records.setdefault(ip, {
                '3xx':0,
                '4xx':0,
                '5xx':0,
                'qts':0,
                'post':0,
                'body_bytes':0,
                'ua':"",
                "lasttime":"",
                })
            ctime = parse_time(date, month, year, day_time)
            IP['body_bytes'] += body_bytes
            IP['qts'] += 1
            if status>=500:
                IP['5xx'] +=1
            elif status>=400:
                IP['4xx'] +=1
            elif status>=400:
                IP['4xx'] +=1
            
            if method=='POST': IP['post']+=1
            if agent and agent!='-': IP['ua'] = agent
            IP['lasttime'] = ctime
            
            #print ctime
            #break
        return self.records
    
    def sort(self, key=None):
        if key in ('lastime','qts','body_bytes','3xx','4xx','5xx', 'post') :
            L = sorted(self.records.items(), lambda x, y: cmp(x[1][key], y[1][key]), reverse=True)
        
        for e in L:
            if e[1]['qts']==1: continue
            #print e

if __name__ == "__main__":
    o = AccessLog(r'C:\Users\liuyouhua\Documents\Tencent Files\53133259\FileRecv\uwv5.com (15).log')
    o.ngx()
    o.sort('body_bytes')
    
