# -*- coding: utf-8 -*-
"""
Python代码示例
"""
import base64
import datetime
from hashlib import sha1
import hmac
import json 
import time

import requests

# openapi地址
OPENAPI_BASE_URL = "https://api.su.baidu.com/%s"

# 签名算法
SIGN_METHOD = "HMAC-SHA1"


def get_signature(sec_key, text):
    """
    计算签名
    """
    hmac_code = hmac.new(sec_key.encode(), text.encode(), sha1).digest()
    return base64.b64encode(hmac_code).decode()


def get_inited_common_params(path):
    """
    获取通用参数
    """
    param_map = {}

    auth_timestamp = str(int(time.time()))
    param_map['X-Auth-Access-Key'] = BaiDuSu.ACCESS_KEY
    param_map['X-Auth-Nonce'] = auth_timestamp
    param_map['X-Auth-Path-Info'] = path
    param_map['X-Auth-Signature-Method'] = SIGN_METHOD
    param_map['X-Auth-Timestamp'] = auth_timestamp

    return param_map


def smart_str(target_string):
    """
    处理字符串
    """
    try:
        if isinstance(target_string, unicode):
            return target_string.encode("utf-8")
        elif isinstance(target_string, str):
            return target_string
    except:
        pass
    
    return json.dumps(target_string)


def get_parsed_all_params(params):
    """
    拼接参数
    """
    keys = params.keys()
    try:
        keys.sort()
    except:
        keys = list(keys)
        keys.sort()
    return "&".join(["%s=%s" % (str(k), smart_str(params[k])) for k in keys])


def get_request_header(path, query_params, body_params):
    """
    获取请求的Header
    """
    common_params = get_inited_common_params(path)
    all_params = {}
    headers = {}

    headers.update(common_params)

    all_params.update(common_params)
    all_params.update(query_params)
    all_params.update(body_params)

    all_params_str = get_parsed_all_params(all_params)

    sign = get_signature(BaiDuSu.SECRET_KEY, all_params_str)

    headers["X-Auth-Sign"] = sign

    return headers


def send_http_request(method='GET', url='', params=None, payload=None, headers=None):
    """
    发送Http请求
    """
    if params is None:
        params = {}
    if payload is None:
        payload = {}
    if headers is None:
        headers = {}
        
    #print '%s:%s' % (method, url)
    #print 'Params:%s' % json.dumps(params, indent=4)
    #print 'Payload:%s' % json.dumps(payload, indent=4)
    #print 'Headers:%s' % json.dumps(headers, indent=4)
        
    resp = requests.request(method, url, params=params, data=json.dumps(payload), headers=headers).json()
    print(resp)
    #print 'Response:%s' % json.dumps(json.loads(resp.text), indent=4)
    
    return resp


class BaiDuSu:
    ACCESS_KEY = ""
    SECRET_KEY = ""
    
    @classmethod
    def waf(cls, domain, mode='on'):
        path = "v3/yjs/zones/settings/waf"
        params = {}

        payload = {}
        payload['domain'] = domain
        payload['value'] = mode #'on','off'
        headers = get_request_header(path, params, payload)
        url = OPENAPI_BASE_URL % path
        response = send_http_request("PATCH", url, params=params, payload=payload, headers=headers)
        return response
    
    @classmethod
    def api_under_attack(cls, domain, mode="low"):
        """
        示例 under_attack => 强力防护high => 高medium => 中low => 低off => 关闭
        """
        path = "v3/yjs/zones/settings/security_level"
        params = {}
        payload = {}
        payload['domain'] = domain
        payload['value'] = mode
        headers = get_request_header(path, params, payload)
        url = OPENAPI_BASE_URL % path
        response = send_http_request("PATCH", url, params=params, payload=payload, headers=headers)
        return response

        
    @classmethod
    def firewall(cls, domain, ip, mode='block'):
        # 模式（mode）有 block, challenge, whitelist, js_challenge
        path = "v3/yjs/zones/settings/firewall"
        params = {}

        payload = {}
        payload['domain'] = domain
        payload['value'] = ip #'100.100.0.0/16'
        payload['operation'] = mode

        headers = get_request_header(path, params, payload)

        url = OPENAPI_BASE_URL % path
        
        response = send_http_request("POST", url, params=params, payload=payload, headers=headers)
        return response



if __name__ == '__main__':
    #BaiDuSu.waf('mufei.ltd')
    #BaiDuSu.firewall('mufei.ltd','192.168.100.100')
    BaiDuSu.api_under_attack('mufei.ltd', mode='under_attack')
    